/*
 * WScout - Lightweight PCAP visualizer.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */

#ifndef WSCOUT_GUI_HEADER_BASE_HH_
# define WSCOUT_GUI_HEADER_BASE_HH_

# include "wscout_gui_abstract_header.hh"

namespace wscout
{

  namespace gui
  {

    /**
     * Base class for all %protocol header widgets.
     *
     * This class provides convenience functions for %protocol header
     * widgets.  Subclass it so you can use registerProtocol() to
     * register your %protocol to Packet.
     *
     * @c Proto should be your %protocol identifier.
     *
     * @c Exact should be the exact type of the subclass.  @e e.g.  @c
     * IPHeader should inherit @c %HeaderBase<IPHeader, @c protocol::ip>.
     * This is an instance of the "Barton-Nackman trick".
     *
     * @see protocol::id, AbstractHeader
     *
     * @todo Should inherit @b both AbstractHeader and HeaderFrame.
     */
    template <class Exact, unsigned Proto>
    struct HeaderBase: public AbstractHeader
    {
      enum
	{
	  protocol = Proto	///< Protocol identifier.
	};

      /**
       * Default constructor.
       *
       * Subclasses @b must provide a default constructor.
       */
      HeaderBase(QWidget* parent = 0);

      /// Allocate an instance of @c Exact using @c new.
      static AbstractHeader*	allocator();

      /// Register allocator() for protocol.
      static void		registerProtocol();
    };


  } // End of namespace gui.

} // End of namespace wscout.

# include "wscout_gui_header_base.hxx"

#endif // ! WSCOUT_GUI_HEADER_BASE_HH_
