/*
 * WScout - Lightweight PCAP visualizer.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */

#ifndef WSCOUT_GUI_PCAP_HEADER_HH_
# define WSCOUT_GUI_PCAP_HEADER_HH_

# include <utility>

# include <QtGui/QLabel>

# include "wscout_gui_header_frame.hh"
# include "wscout_pcap_trace.hh"
# include "wscout_protocol_ids.hh"

namespace wscout
{

  namespace gui
  {

    /**
     * PCAP header widget.
     *
     * Does not inherit AbstractHeader nor HeaderBase.  That for several
     * reasons.
     *
     * <ul>
     *
     *   <li>Raw PCAP headers lack information such as sequence number
     *       and link type.  Therefore it is impossible to satisfy the
     *       AbstractHeader interface.</li>
     *
     *   <li>Since we are parsing PCAP files, we are sure to have
     *       a PCAP header in each packet.  Therefore there is no need
     *       to use a protocol registration mechanism as provided by
     *       HeaderBase.</li>
     *
     * </ul>
     *
     */
    class PcapHeader: public HeaderFrame
    {
      Q_OBJECT

    public:
      /// PCAP packet type.
      typedef pcap::Trace::iterator::value_type		packet;

      /// PCAP link type.
      typedef pcap::Trace::link_type			link_type;

      /// Next header description.  @see AbstractHeader::next_header.
      typedef std::pair<protocol::id, const void*>	next_header;

      /// Default constructor.
      PcapHeader(QWidget* parent = 0);

      /// Setup the widget for a given link type, sequence number and packet.
      next_header		setup(link_type		t,
				      unsigned		n,
				      const packet&	p);

      /// Retrieve the next header's description.
      static next_header	nextHeader(link_type t, const packet& p);

    private:
      QLabel	id_;
      QLabel	date_;
      QLabel	len_;
    };

  } // End of namespace gui.

} // End of namespace wscout.

#endif // ! WSCOUT_GUI_PCAP_HEADER_HH_
