/*
 * WScout - Lightweight PCAP visualizer.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */

extern "C"
{
# include <inttypes.h>
}

#include <boost/format.hpp>
#include <boost/lexical_cast.hpp>

#include <trace-tools/prism_header.hh>

#include "wscout_gui_prism_header.hh"

namespace wscout
{

  namespace gui
  {

    PrismHeader::PrismHeader(QWidget* parent):
      HeaderBase<PrismHeader, protocol::PRISM_HEADER> (parent)
    {
      QBoxLayout* const	l = layout();

      time.setFont(QFont ("Fixed"));
      l->addWidget(&time);
      l->addSpacing(default_spacing);

      l->addWidget(&rate);
      l->addSpacing(default_spacing);

      l->addWidget(&channel);
      l->addStretch();

      l->addWidget(&signal);
    }

    PrismHeader::next_header
    PrismHeader::setup(const void* h, size_t caplen)
    {
      if (caplen < sizeof (prism::header))
	{
	  rate.hide();
	  channel.hide();
	  signal.hide();
	  time.hide();
	  return next_header (protocol::NONE, 0);
	}

      const prism::header* const prism = static_cast<const prism::header*> (h);

      const std::string	timestr =
	boost::lexical_cast<std::string> (boost::format ("%010u:%010u")
					  % prism->hosttime.data
					  % prism->mactime.data);

      setUpdatesEnabled(false);
      time.setText(timestr.c_str());
      rate.setText(tr("%1 Mb/s").arg(prism->rate.data / 2.));
      channel.setText(tr("channel %1").arg(prism->channel.data));
      signal.setText(tr("rssi/signal/noise: %1 / %2 / %3")
		     .arg(prism->rssi.data)
		     .arg(prism->signal.data)
		     .arg(prism->noise.data));
      rate.show();
      channel.show();
      signal.show();
      time.show();
      setUpdatesEnabled(true);

      return next_header (protocol::IEEE802_11, prism + 1);
    }

  } // End of namespace gui.

} // End of namespace wscout.
