/*
 * trace-tools - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef PCAP_FRAME_ITERATOR_HH_
# define PCAP_FRAME_ITERATOR_HH_

# include <trace-tools/pcap/frame_descriptor.hh>
# include <trace-tools/tool/iterator_base.hh>

namespace pcapxx
{

  template <class D>
  struct descriptor;

  /// Internal compounds - not designed for users.
  namespace internals
  {

    template <class D, class B>
    struct frame_iterator;

  } // End of namespace pcapxx::internals.
} // End of namespace pcapxx.

namespace tool
{

    /// Virtual %types for pcapxx::internals::frame_iterator.  @see types
  template <class D, class B>
  struct types< pcapxx::internals::frame_iterator<D, B> >
  {
    typedef pcapxx::frame_descriptor	value_type;
    typedef pcapxx::descriptor<D>	iterable_type;
  };

} // End of namespace tool.

namespace pcapxx
{
  namespace internals
  {

    /// Iterator over frames for @link descriptor PCAP descriptors @endlink.
    template <class Desc, class Bottom = tool::bottom>
    struct frame_iterator:
      tt_inherit(public tool::iterator_base, frame_iterator<Desc, Bottom>)
    {
      typedef tt_get_exact(Bottom, frame_iterator<Desc, Bottom>) exact_type;

      /// @link tool::iterable @endlink -compatible constructor.
      frame_iterator(descriptor<Desc>& desc, bool end);

      /// Retrieve the next frame.
      void		increment();

      template <class D, class B>
      friend
	void
	fill(void*, const pkthdr*, const void*);

    };

    /**
     * Set a frame %iterator's value according to a PCAP header and data.
     *
     * Actually, this function should be a static method of
     * frame_iterator.  GCC does not allow that however since version
     * 4.2.1. (Apparently it refuses to use a static method as template
     * parameter for descriptor::loop.)
     */
    template <class D, class B>
    void
    fill(void*, const pkthdr*, const void*);

  } // End of namespace pcapxx::internals.

} // End of namespace pcapxx.

# include "frame_iterator.hxx"

#endif // ! PCAP_FRAME_ITERATOR_HH_
