/*
 * trace-tools - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef PCAP_FRAME_ITERATOR_HXX_
# define PCAP_FRAME_ITERATOR_HXX_

# include <stdexcept>

# include "frame_iterator.hh"

namespace pcapxx
{

  namespace internals
  {

    template <class D, class B>
    frame_iterator<D, B>::frame_iterator(descriptor<D>& desc, bool end):
      tool::iterator_base<exact_type> (desc, end)
    {
    }

    template <class D, class B>
    void
    frame_iterator<D, B>::increment()
    {
      this->v_ = boost::none_t ();
      if(this->i_->template loop< fill<D, B> >(this, 1))
	throw std::logic_error ("Unknown internal error.");
    }

    template <class D, class B>
    void
    fill(void* it, const pkthdr* h, const void* b)
    {
      static_cast<frame_iterator<D, B>*> (it)->v_ =
        typename frame_iterator<D, B>::exact_type::value_type (h, b);
    }

  } // End of namespace pcapxx::internals.

} // End of namespace pcapxx.

#endif // ! PCAP_FRAME_ITERATOR_HXX_
