/*
 * trace-tools - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef TOOL_ENDIANNESS_HH_
# define TOOL_ENDIANNESS_HH_

extern "C"
{
# include <inttypes.h>
}

namespace tool
{
  /// Consider @c p as a pointer to unsigned shorts and return *p.
  uint16_t
  extract_unswapped_short(const void* p);

  /// Return @c s with swapped low and high bytes.
  uint16_t
  extract_swapped_short(const uint16_t s);

  /// Consider @c p as a pointer to endian-swapped unsigned shorts and
  ///  return *p.
  uint16_t
  extract_swapped_short(const void* p);

  /// Return @c s, possibly with endianness adjustment if @c swapped.
  uint16_t
  extract_short(const uint16_t s, bool swapped);

  /// Return @c *p, possibly with endianness adjustment if @c swapped.
  uint16_t
  extract_short(const void* p, bool swapped);

  /// Consider @c p as a pointer to big-endian unsigned shorts and return *p.
  uint16_t
  extract_big_endian_short(const void* p);

} // End of namespace tool.

# include "endianness.hxx"

#endif // ! TOOL_ENDIANNESS_HH_
