/*
 * trace-tools - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef TOOL_ENDIANNESS_HXX_
# define TOOL_ENDIANNESS_HXX_

#include "endianness.hh"

namespace tool
{

  inline
  uint16_t
  extract_unswapped_short(const void* p)
  {
    return *static_cast<const uint16_t*> (p);
  }

  inline
  uint16_t
  extract_swapped_short(const uint16_t s)
  {
    return (s >> 8) | (s << 8);
  }

  inline
  uint16_t
  extract_swapped_short(const void* p)
  {
    return extract_swapped_short(extract_unswapped_short(p));
  }

  inline
  uint16_t
  extract_short(const uint16_t s, bool swapped)
  {
    return swapped ? extract_swapped_short(s) : s;
  }


  inline
  uint16_t
  extract_short(const void* p, bool swapped)
  {
    return swapped ? extract_swapped_short(p) : extract_unswapped_short(p);
  }

  inline
  uint16_t
  extract_big_endian_short(const void *p)
  {
    const uint8_t *const c = static_cast<const uint8_t*> (p);

    return uint16_t (*c) | uint16_t (*(c + 1)) << 8;
  }

} // End of namespace tool.

#endif // ! TOOL_ENDIANNESS_HXX_
