/*
 * trace-tools - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef TOOL_LINEAR_REGRESSION_HH_
# define TOOL_LINEAR_REGRESSION_HH_

# include <utility>

namespace tool
{

  /**
   * Perform a linear regression.
   *
   * Input data is provided by @c start and @c last.  @c
   * InputIterator::value_type must have a @c first and @c second
   * attribute.  @c ImplType is used as the algorithm's underlying
   * implementation type.
   *
   * @return a pair of coefficients.
   */
  template <class ImplType, class InputIterator>
  std::pair<ImplType, ImplType>
  linear_regression(const InputIterator& start,
		    const InputIterator& last);

} // End of namespace tool.

# include "linear_regression.hxx"

#endif // ! TOOL_LINEAR_REGRESSION_HH_
