/*
 * trace-tools - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_FILTER_UNIQUELY_IDENTIFIABLE_HXX_
# define WIFI_FRAME_FILTER_UNIQUELY_IDENTIFIABLE_HXX_

# include <trace-tools/wifi/frame/dissector/dissector.hh>

namespace wifi
{
  namespace frame
  {
    namespace filter
    {

      namespace internals
      {

	template <class UniqueId>
	ui_iterator_value_type<UniqueId>::
	ui_iterator_value_type(const UniqueId&			i,
			       const tool::microseconds&	m):
	  UniqueId (i),
	  microseconds_ (m)
	{
	}

	template <class UniqueId>
	const tool::microseconds&
	ui_iterator_value_type<UniqueId>::microseconds() const
	{
	  return microseconds_;
	}

	template <class U, class I, class H, class B1, class B2>
	ui_iterator<U, I, H, B1, B2>::ui_iterator(iterable_type&	uif,
						  bool			end):
	  tool::iterator_base<exact_type> (uif, end)
	{
	}

	template <class U, class I, class H, class B1, class B2>
	void
	ui_iterator<U, I, H, B1, B2>::increment()
	{
	  typedef typename exact_type::value_type  value_type;

	  uniquely_identifiable<U, I, H, B1>&	iterable = *(this->i_);
	  I&					next = iterable.next_;
	  boost::optional<U>			id;

	  while (next != iterable.end_ and not id)
	    {
	      const H* h = reinterpret_cast<const H*> (next->bytes().get());

	      iterable.mactime_.tick(h, next->pcap_header()->caplen);
	      id = make_unique_id<U, H>(*next, *iterable.mapping_,
					iterable.
					  prism_header_noise_means_error);
	      ++next;
	    }
	  if (id)
	    this->v_ =
	      typename exact_type::value_type (id.get(), iterable.mactime_);
	  else
	    this->v_ = boost::none_t ();
	}

      } // End of namespace wifi::frame::filter::internals.

      template <class U, class I, class H, class B>
      uniquely_identifiable<U, I, H, B>::uniquely_identifiable(const I& begin,
							       const I& end,
							       addr_mapping&
							         mapping):
	prism_header_noise_means_error (false),
	next_ (begin),
	end_ (end),
	mapping_ (&mapping),
	mactime_ ()
      {
      }

    } // End of namespace wifi::frame::filter.

  } // End of namespace wifi::frame.

} // End of namespace wifi.

#endif // ! WIFI_FRAME_FILTER_UNIQUELY_IDENTIFIABLE_HXX_
