/*
 * trace-tools - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#include <iostream>
#include <stdexcept>
#include <cstdlib>

#include "config.hh"

#include <trace-tools/pcap/descriptor.hh>
#include <trace-tools/wifi/addr_mapping.hh>
#include <trace-tools/wifi/frame/source_timestamp_id.hh>
#include <trace-tools/wifi/frame/filter/uniquely_identifiable.hh>
#include <trace-tools/wifi/frame/filter/intersector.hh>

int
main(int argc, char *const argv[])
{
  if (argc != 3 and argc != 4)
    {
      std::cerr << "Usage:\t"  << argv[0] << " <trace1> <trace2> [mapping]"
		<< std::endl;
      return EXIT_FAILURE;
    }

  try
    {
      typedef wifi::frame::filter::
	uniquely_identifiable<wifi::frame::source_timestamp_id,
			      pcapxx::descriptor<>::iterator>
	uniquely_identifiable;
      typedef
	wifi::frame::filter::intersector<uniquely_identifiable::iterator,
					 uniquely_identifiable::iterator>
	intersector;

      wifi::addr_mapping	m (argc == 4 ? argv[3] : "mapping");
      pcapxx::descriptor<>	d1 (argv[1]);
      pcapxx::descriptor<>	d2 (argv[2]);
      uniquely_identifiable	u1 (d1.begin(), d1.end(), m);
      uniquely_identifiable	u2 (d2.begin(), d2.end(), m);

      // FIXME: Pass as a program argument.
      u1.prism_header_noise_means_error = true;
      u2.prism_header_noise_means_error = true;

      intersector		i (u1.begin(), u1.end(), u2.begin(), u2.end());

      for (intersector::iterator j = i.begin(); j != i.end(); ++j)
	std::cout << j->first << '\t' << j->second << '\n';
    }
  catch (const std::exception& e)
    {
      std::cerr << "ERROR: " << e.what() << '.' << std::endl;
      return EXIT_FAILURE;
    }
}
