/*
 * trace-tools - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#include <iostream>
#include <fstream>
#include <stdexcept>
#include <cstdlib>

#include "config.hh"

#include <trace-tools/pcap/descriptor.hh>
#include <trace-tools/wifi/addr_mapping.hh>
#include <trace-tools/wifi/frame/source_timestamp_id.hh>
#include <trace-tools/wifi/frame/filter/uniquely_identifiable.hh>

namespace
{

  std::string
  mapping(const std::string unique_frames_filename)
  {
      return unique_frames_filename + ".mapping";
  }

} // End of anonymous namespace.

int
main(int argc, char *const argv[])
{
  if (argc != 2 and argc != 3)
    {
      std::cerr << "Usage:\t"  << argv[0]
		<< " <unique_frames_file> [mapping_file]"
		<< std::endl;
      return EXIT_FAILURE;
    }

  try
    {
      typedef wifi::frame::filter::
	uniquely_identifiable<wifi::frame::source_timestamp_id,
			      pcapxx::descriptor<>::iterator>
	uniquely_identifiable;

      wifi::addr_mapping	m (argc == 3 ? argv[2] : mapping(argv[1]));
      pcapxx::descriptor<>	d ("-");
      uniquely_identifiable	u (d.begin(), d.end(), m);
      std::ofstream		o (argv[1]);

      // FIXME: Pass as a program argument.
      u.prism_header_noise_means_error = true;

      for (uniquely_identifiable::iterator i = u.begin(); i != u.end(); ++i)
	o << *i << '\n';
    }
  catch (const std::exception& e)
    {
      std::cerr << "ERROR: " << e.what() << '.' << std::endl;
      return EXIT_FAILURE;
    }
}
