/*
 * trace-tools - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#include <iostream>
#include <string>
#include <cstdlib>
#include <cstring>

#include <trace-tools/pcap/descriptor.hh>

int
main()
{
  std::cerr << "WARNING: This test requires several giga-bytes "
    " available on your disk." << std::endl
	    << "WARNING: Set the DONT_RUN_THE_BIG_PCAP_TEST environment "
    "variable to disable it." << std::endl;

  if (getenv("DONT_RUN_THE_BIG_PCAP_TEST"))
    return EXIT_SUCCESS;

  const std::string	pcapname = "big";
  const size_t		npackets = (1 << 26);
  const size_t		snaplen = 16;

  std::cout << "Creating a big Pcap file..." << std::endl;
  {
    char		junk[snaplen];
    pcapxx::pkthdr	hdr;

    memset(junk, 0, snaplen);
    hdr.ts.tv_sec = 0;
    hdr.ts.tv_usec = 0;
    hdr.caplen = snaplen;
    hdr.len = snaplen;


    pcapxx::descriptor<>	desc (DLT_PRISM_HEADER, snaplen);
    pcapxx::dumper		dump = desc.dumper(pcapname);

    for (unsigned i = 0; i < npackets; ++i)
      {
	++hdr.ts.tv_sec;
	dump (&hdr, junk);
      }
  }

  std::cout << "Checking whether we are capable of opening it or not..."
	    << std::endl;
  pcapxx::descriptor<>	test (pcapname);
  int			n = 0;

  std::cout << "Integrity check..." << std::endl;
  for (pcapxx::descriptor<>::iterator i = test.begin(); i != test.end(); ++i)
    if (i->pcap_header()->ts.tv_sec != ++n)
      return EXIT_FAILURE;
}
